using AngularNote.Models;
using DotNetNote.Common;
using DotNetNote.Components;
using DotNetNote.Models;
using DotNetNote.Models.Companies;
using DotNetNote.Models.Exams;
using DotNetNote.Models.RecruitManager;
using DotNetNote.Services;
using DotNetNote.Settings;
using DotNetNoteCom.Models;
using GoodsManager.Models;
using MemberManagement.Data;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Routing;
//using Microsoft.AspNetCore.Http; // ASP.NET Core 
using Microsoft.AspNetCore.SpaServices.Webpack;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
//using Microsoft.Extensions.FileProviders;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;
using Serilog;
using System;
using System.IO;
using System.Text;

namespace DotNetNoteCom
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            //[!] Configuration: JSON  ͸ POCO Ŭ 
            services.Configure<DotNetNoteSettings>(
                Configuration.GetSection("DotNetNoteSettings"));

            //[!] ͸ ¡  (ɼ)
            services.AddDirectoryBrowser();


            // <TodoComponent>
            //[1] -޸ DB   
            //    TodoContext   Ŭ  DB CRUD  
            services.AddDbContext<TodoContext>(
                options => options.UseInMemoryDatabase("TodoComponent"));
            ////[2] SQL Server  (PM> Add-Migration -> PM> Update-Database)
            //services.AddDbContext<TodoContext>(
            //    options => options.UseSqlServer(Configuration["ConnectionString"]));
            // </TodoComponent>


            //[!]  ޸ ͺ̽   
            services.AddDbContext<AccountContext>(opt => opt.UseInMemoryDatabase("AccountContext"));


            //[] AspNetCore21_MemberManagement_EF Core ȸ   ϱ
            services.AddDbContext<MemberDbContext>(options =>
                options.UseSqlServer(
                    Configuration.GetConnectionString("DefaultConnection")));


            //<  SQL Server ϱ  >
            services.AddDistributedSqlServerCache(options =>
            {
                options.ConnectionString = Configuration["ConnectionString"];
                options.SchemaName = "dbo";
                options.TableName = "SQLSessions";
            });
            //</  SQL Server ϱ  >



            //[DNN] TempData[] ü 
            // In-Memory ĳ 
            services.AddMemoryCache();

            //[!]  ü : Microsoft.AspNetCore.Session.dll
            //services.AddSession(); 
            // Session ü  ɼ ο 
            services.AddSession(options =>
            {
                //   ð
                options.IdleTimeout = TimeSpan.FromMinutes(30);
            });



            #region ASP.NET Core 2.0 Ű : ConfigureServices()
            //[!] ASP.NET Core 2.0 Ű : ܼ
            //services.AddAuthentication("Cookies").AddCookie(options => { options.AccessDeniedPath = 
            //    "/User/Forbidden/"; options.LoginPath = "/User/Login/"; });

            //[!] ASP.NET Core 2.0 Ű   JWT : ⺻
            //services.AddAuthentication()
            services.AddAuthentication(options =>
            {
                options.DefaultAuthenticateScheme =
                    CookieAuthenticationDefaults.AuthenticationScheme;
                options.DefaultSignInScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                options.DefaultChallengeScheme = CookieAuthenticationDefaults.AuthenticationScheme;
            })
            //services.AddAuthentication(options =>
            //{
            //    options.DefaultAuthenticateScheme =
            //        JwtBearerDefaults.AuthenticationScheme;
            //    options.DefaultSignInScheme = JwtBearerDefaults.AuthenticationScheme;
            //    options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
            //})
            .AddCookie(options =>
            {
                options.LoginPath = new PathString("/User/Login");
                options.AccessDeniedPath = new PathString("/User/Forbidden");
                options.SlidingExpiration = true;
            })
            .AddJwtBearer(JwtBearerDefaults.AuthenticationScheme, options =>
            {
                options.RequireHttpsMetadata = false;
                options.SaveToken = true;
                options.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateAudience = false,
                    ValidateIssuer = false,
                    ValidateIssuerSigningKey = true,
                    // Ű ڿ   
                    IssuerSigningKey = new SymmetricSecurityKey(
                        Encoding.UTF8.GetBytes(
                            Configuration["SymmetricSecurityKey"])),
                    ValidateLifetime = true,
                    ClockSkew = TimeSpan.FromMinutes(5)
                };
            });
            #endregion



            //[User][9] Policy 
            services.AddAuthorization(options =>
            {
                // Users Role , Users Policy ο
                options.AddPolicy(
                    "Users", policy => policy.RequireRole("Users"));
                // Users Role ְ UserId "Admin"̸ "Administrators" ο
                options.AddPolicy(
                    "Administrators",
                        policy => policy
                            .RequireRole("Users")
                            .RequireClaim("UserId", // ҹ 
                                Configuration
                                    .GetSection("DotNetNoteSettings")
                                    .GetSection("SiteAdmin").Value)
                            );
            });


            //[CORS] CORS 
            services.AddCors(options =>
            {
                options.AddPolicy("AllowAnyOrigin",
                    builder => builder
                    .AllowAnyOrigin()
                    .AllowAnyMethod()
                    .AllowAnyHeader());
            });


            //[MVC] MVC  ߰  JSON  ɼ 
            //services.AddMvc()
            //    .AddXmlDataContractSerializerFormatters(); // XML Web API ᱣ 
            services.AddMvc(); // ASP.NET Core  ߿ 
                               //.AddJsonOptions(options =>
                               //{
                               //    //[Web API] JSON Ӽ ù ҹ: ASP.NET Core 1.0 ⺻
                               //    options.SerializerSettings.ContractResolver =
                               //    new CamelCasePropertyNamesContractResolver();
                               //});
            #region services.AddMvc() ߰ ɼǵ ()
            ////[!] Web API JSON  빮( Ŭ ) 
            //services.AddMvc().AddJsonOptions(
            //    option => option.SerializerSettings.ContractResolver =
            //        new Newtonsoft.Json.Serialization.DefaultContractResolver());

            ////[!]  JSON ùڸ 빮ڷ ǥ(?)
            //services.AddMvc() 
            //    .AddJsonOptions(options =>
            //    {
            //        if (options.SerializerSettings.ContractResolver != null)
            //        {
            //            //[Web API] JSON Ӽ ù 빮
            //            var castedResolver = 
            //                options.SerializerSettings.ContractResolver 
            //                    as Newtonsoft.Json.Serialization
            //                        .DefaultContractResolver;
            //            //  Ŭ 빮ڷ Ǿ ״ 
            //            castedResolver.NamingStrategy = null; 
            //        }
            //    });

            ////[!]   , JSON ſ XML ȯϰ Ѵٸ Ʒ ڵ 
            //services.AddMvc(options =>
            //{
            //    options.OutputFormatters.Add(new XmlDataContractSerializerOutputFormatter());
            //}); 
            #endregion

            //[DI]  (Dependency Injection)
            DependencyInjectionContainer(services);
        }

        /// <summary>
        ///    ڵ常  Ƽ 
        /// - 丮 
        /// </summary>
        private void DependencyInjectionContainer(IServiceCollection services)
        {
            //[?] ConfigureServices ȣǱ  DI(Ӽ )  ʽϴ.

            //[DNN][!] Configuration ü : 
            //    IConfiguration Ǵ IConfigurationRoot Configuration ü 
            //    appsettings.json  ͺ̽  ڿ 
            //    丮 Ŭ   ֵ 
            // IConfiguration  -> Configuration νϽ  
            services.AddSingleton<IConfiguration>(Configuration);


            //[Demo] DemoFinder  : ⺻  DI ̳ 
            services.AddTransient<DotNetNote.Models.DataFinder>();

            //[DI] InfoService Ŭ  : 30.3
            services.AddSingleton<InfoService>();
            services.AddSingleton<IInfoService, InfoService>();

            //[DI(Dependency Injection)]  : 30.4
            //services.AddTransient<CopyrightService>();
            services.AddTransient<ICopyrightService, CopyrightService>();

            //[DI] @inject Ű 信  Ŭ Ӽ Ǵ ޼  
            services.AddSingleton<CopyrightService>();

            //[DI] GuidService 
            services.AddTransient<GuidService>();
            services.AddTransient<IGuidService, GuidService>();
            services.AddTransient<ITransientGuidService, TransientGuidService>();
            services.AddScoped<IScopedGuidService, ScopedGuidService>();
            services.AddSingleton<ISingletonGuidService, SingletonGuidService>();


            //[User][5] ȸ 
            services.AddTransient<IUserRepository, UserRepository>();
            // LoginFailedManager
            services.AddTransient<ILoginFailedRepository, LoginFailedRepository>();
            services.AddTransient<ILoginFailedManager, LoginFailedManager>();


            //[CommunityCamp]   
            services.AddTransient<ICommunityCampJoinMemberRepository,
                CommunityCampJoinMemberRepository>();


            //[DNN][1] Խ               
            //[1] ڿ ڿ  ڿ 
            //services.AddSingleton<INoteRepository>(
            //  new NoteRepository(
            //      Configuration["Data:DefaultConnection:ConnectionString"]));            
            //[2]   Configuration 
            //[a] NoteRepository IConfiguration ͺ̽  ڿ 
            services.AddTransient<INoteRepository, NoteRepository>();
            //[b] CommentRepository ڿ ͺ̽ Ṯڿ  
            //services.AddSingleton<INoteCommentRepository>(
            //    new NoteCommentRepository(
            //        Configuration["ConnectionStrings:DefaultConnection"]));
            //[b]  ڵ带 Ʒ ڵ 
            services.AddTransient<INoteCommentRepository, NoteCommentRepository>();


            services.AddTransient<MaximServiceRepository, MaximServiceRepository>();

            //[Tech]  
            services.AddTransient<ITechRepository, TechRepository>();

            //[Attendee]   -  
            services.AddTransient<IAttendeeRepository, AttendeeRepository>();

            //[RecruitManager] 
            services.AddTransient<
                IRecruitSettingRepository, RecruitSettingRepository>();
            services.AddTransient<IRecruitRegistrationRepository,
                RecruitRegistrationRepository>();

            //[IdeaManager]: ̵ : ASP.NET Core Angular A to Z 
            services.AddTransient<IIdeaRepository, IdeaRepository>();

            //[One], [Two], ... 
            services.AddTransient<IOneRepository, OneRepository>();
            services.AddTransient<ITwoRepository, TwoRepository>();
            services.AddTransient<IThreeRepository, ThreeRepository>();
            services.AddTransient<IFourRepository, FourRepository>();
            services.AddTransient<IFiveRepository, FiveRepository>();

            // Exams: Questions, ... 
            services.AddTransient<IQuestionRepository, QuestionRepository>();

            //[HeroManager]:  : ASP.NET Core Angular A to Z
            services.AddTransient<IHeroRepository, HeroRepository>();

            // ĳ 
            services.AddTransient<ICharacterRepository, CharacterRepository>();

            // Ʈ 
            //services.AddTransient<IPointRepository, PointRepository>(); // DB 
            services.AddTransient<IPointRepository, PointRepositoryInMemory>(); // -޸ 
            services.AddTransient<IPointLogRepository, PointLogRepository>();

            //     Ʈ
            services.AddTransient<IUserModelRepository, UserModelRepository>();

            // DotNetSale
            services.AddTransient<ICategoryRepository, CategoryRepositoryInMemory>();

            // <GoodsManager>
            // IGoodsRepository  -> GoodsRepository νϽ 
            services.AddTransient<IGoodsRepository, GoodsRepository>();
            // </GoodsManager>

            //  ذ: ISignRepository => SignRepositoryInMemory
            services.AddTransient<ISignRepository, SignRepositoryInMemory>();

            services.AddSingleton<ICompanyRepository>(new CompanyRepositoryAdo(Configuration["ConnectionStrings:DefaultConnection"]));
            //services.AddSingleton<ICompanyRepository>(new CompanyRepositoryDapper(Configuration["ConnectionStrings:DefaultConnection"]));
            //services.AddTransient<ICompanyRepository, CompanyRepositoryEntityFramework>();
        }


        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(
            IApplicationBuilder app,
            IHostingEnvironment env,
            ILoggerFactory loggerFactory,
            IServiceProvider serviceProvider)
        {

            //[!] ⺻  α
            loggerFactory.AddConsole(Configuration.GetSection("Logging"));
            loggerFactory.AddDebug();
            //loggerFactory.AddConsole(LogLevel.Debug);
            //loggerFactory.AddDebug(LogLevel.Debug);

            //[0]  : 
            //  HTTP ο ʿ    ̵ ߰ؼ 
            if (env.IsDevelopment())
            {
                //[!]  ߻   ڼ  ǥ
                app.UseDeveloperExceptionPage(); //[]

                // 1.1  ũ   Ȩ ڰŸ   
                //app.UseBrowserLink(); 

                // ޱַ ϱ    ̵ ߰
                app.UseWebpackDevMiddleware(new WebpackDevMiddlewareOptions
                {
                    HotModuleReplacement = true
                });
            }
            else
            {
                ////[!] SSL 
                //app.Use(async (context, next) => {
                //    if (context.Request.IsHttps)
                //    {
                //        await next();
                //    }
                //    else
                //    {
                //        var withHttps =
                //            "https://"
                //            + context.Request.Host
                //            + context.Request.Path;
                //        context.Response.Redirect(withHttps); 
                //    }
                //});

                app.UseExceptionHandler("/Home/Error");
            }




            // <TodoComponent>
            //[!] IServiceProvider Ͽ ӽ  ϱ
            var todo = serviceProvider.GetService<TodoContext>();
            todo.Todos.Add(new Todo { Id = -2, Title = "Angular", IsDone = false });
            todo.Todos.Add(new Todo { Id = -1, Title = "ASP.NET Core", IsDone = true });
            todo.SaveChanges();
            // </TodoComponent>





            //[DNN] TempData ü 
            app.UseSession(); //[!]  ü , ݵ UseMvc()  ȣǾ 

            // ̵ ߰

            ////   ̵ ߰
            //app.Use(async (ctx, next) => {
            //    Console.WriteLine("Hello pipeline, {0}", ctx.Request.Path);
            //    await next();
            //});

            //[!]    ̵ 
            //app.UseWelcomePage(); // ϳ   

            //[!] MIME Ÿ  : FileExtensionContentTypeProvider Ŭ 

            //[!]   ̵ ߰

            //[!] UseDefaultFiles() ̵: ⺻  
            //DefaultFilesOptions options = new DefaultFilesOptions();
            //options.DefaultFileNames.Clear();
            //options.DefaultFileNames.Add("NewDefault.html");
            //app.UseDefaultFiles(options);

            //// #### Azure Web Apps   #### 
            ////[!]    ߰ : Azure Web Apps  
            //// Microsoft.AspNetCore.StaticFiles  ߰
            //// [a] wwwroot HTML/CSS/JavaScript/Images    
            //app.UseStaticFiles(
            //    new StaticFileOptions()
            //    {
            //        FileProvider = new PhysicalFileProvider(
            //            Path.Combine(Directory.GetCurrentDirectory()
            //                , @"MyStaticFiles")),
            //        RequestPath = new PathString("/StaticFiles")
            //    }
            //);

            ////[b] Ư θ /ImageDir/  ؼ ϰ 
            //app.UseStaticFiles(
            //    new StaticFileOptions()
            //    {
            //        FileProvider = new PhysicalFileProvider(@"C:\Temp"),
            //        RequestPath = new PathString("/ImageDir")
            //    }
            //);

            //[!] UseFileServer() ̵ :    ͸ ¡ ǥ  
            app.UseFileServer(); // Ʒ 3 ̵ 
                                 //[!]     UseStaticFiles() ̵ ߰
                                 //app.UseStaticFiles(); //[]
                                 //app.UseDirectoryBrowser();
                                 //app.UseDefaultFiles(); 

            //[!]  ڵ ǥ 
            app.UseStatusCodePages(); //[] Status Code: 404; Not Found ·  ǥð µ

            //[User][1] Ű    ڵ: 
            // Microsoft.AspNetCore.Authentication.Cookies Ű() ߰
            // ASP.NET Core 1.X Ű 
            //app.UseCookieAuthentication(
            //    new CookieAuthenticationOptions()
            //    {
            //        AuthenticationScheme = "Cookies",
            //        // α ʾ  [Authorize] ؼ Login ̵
            //        LoginPath = new PathString("/User/Login"),
            //        AccessDeniedPath = new PathString("/User/Forbidden"),
            //        AutomaticAuthenticate = true,
            //        AutomaticChallenge = true
            //    }
            //);


            #region ASP.NET Core 2.0 Ű : Configure()
            // ASP.NET Core 2.0 Ű 
            app.UseAuthentication();
            #endregion


            // 31.8.4. Serilog
            Log.Logger = new LoggerConfiguration()
                .MinimumLevel.Debug()
                //.WriteTo.RollingFile(Path.Combine(env.ContentRootPath, "DnnLogs-{Date}.txt"))
                .WriteTo.RollingFile(Path.Combine(env.ContentRootPath + "\\Logs", "DnnLogs-{Date}.txt"))
                .CreateLogger();
            loggerFactory.AddSerilog();


            //[!] CORS
            //app.UseCors(options => options.WithOrigins(
            //    "http://dotnetnote.azurewebsites.net/api/values"));
            app.UseCors(
                options => options.AllowAnyOrigin().WithMethods("GET"));


            //[CORS] CORS 
            //app.UseCors("AllowAnyOrigin");


            // α׾ƿ ó
            app.UseRouter(routes =>
            {
                // ~/Logout  ûϸ ڵ α׾ƿ
                routes.MapGet("Logout", async context =>
                {
                    await context.SignOutAsync("Cookies");
                    context.Response.Redirect("/");
                });
                routes.MapPost("Logout", async context =>
                {
                    await context.SignOutAsync("Cookies");
                    context.Response.Redirect("/");
                });
            });


            //[MVC] 
            //[1] MVC ⺻ Ʈ 
            //app.UseMvcWithDefaultRoute(); //[]

            //[2] MVC   Ʈ  
            app.UseMvc(routes =>
            {
                //[1] ASP.NET Core   
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Home}/{action=Index}/{id?}");

                routes.MapSpaFallbackRoute(
                    name: "spa-fallback",
                    defaults: new { controller = "Home", action = "Index" });

                ////[2] Angular   
                //routes.MapRoute(
                //    name: "default",
                //    template: "{controller=Angular}/{action=Index}/{id?}");

                //routes.MapSpaFallbackRoute(
                //    name: "spa-fallback",
                //    defaults: new { controller = "Angular", action = "Index" });

                ////[3] Blog   
                //routes.MapRoute(
                //    name: "default",
                //    template: "{controller=Cloud}/{action=Index}/{id?}");

                //routes.MapSpaFallbackRoute(
                //    name: "spa-fallback",
                //    defaults: new { controller = "Cloud", action = "Index" });
            });

            //[!] ζ ̵
            app.Run(async (context) =>
            {
                // ѱ 
                context.Response.Headers["Content-Type"] = "text/html; charset=utf-8";
                await context.Response.WriteAsync("ȳϼ.", System.Text.Encoding.UTF8);
            });
        }
    }
}
